import { QueryClient, QueryClientProvider } from '@tanstack/react-query';
import React from 'react';
import { http } from 'viem';
import { abstractTestnet } from 'viem/chains';
import { createConfig, WagmiProvider } from 'wagmi';
import { abstractWalletConnector } from './abstractWalletConnector.js';
/**
 * AbstractWalletProvider is a React provider that wraps the WagmiProvider and QueryClientProvider.
 * It provides the AbstractWalletContext to its children.
 * @example
 * ```tsx
 * import { AbstractWalletProvider } from '@abstractwallet/agw-react';
 *
 * const App = () => {
 *   // optional configuration overrides
 *   const config = {
 *     testnet: true,
 *     transport: http("https://your.abstract.node.example.com/rpc")
 *   };
 *   return (
 *     <AbstractWalletProvider config={config}>
 *       <Component {...pageProps} />
 *     </AbstractWalletProvider>
 *   );
 * };
 * ```
 * @param {AbstractWalletConfig} config - The configuration for the AbstractWalletProvider.
 */
export const AbstractWalletProvider = ({ config = {
    testnet: false,
    transport: http(),
}, children, }) => {
    const { testnet = false, transport } = config;
    // TODO: replace with mainnet when we have the configuration
    const chain = testnet ? abstractTestnet : abstractTestnet;
    const wagmiConfig = createConfig({
        chains: [chain],
        ssr: true,
        connectors: [abstractWalletConnector()],
        transports: {
            [chain.id]: transport ?? http(),
        },
        multiInjectedProviderDiscovery: false,
    });
    const queryClient = new QueryClient();
    return (React.createElement(WagmiProvider, { config: wagmiConfig },
        React.createElement(QueryClientProvider, { client: queryClient }, children)));
};
//# sourceMappingURL=agwProvider.js.map